@@ -63,7 +63,7 @@
 #include "extcap_spawn.h"
 
 #ifdef _WIN32
-static HANDLE pipe_h = NULL;
+static HANDLE pipe_h = INVALID_HANDLE_VALUE;
 #endif
 
 static void extcap_child_watch_cb(GPid pid, gint status, gpointer user_data);
@@ -1031,13 +1031,32 @@ void extcap_if_cleanup(capture_options *capture_opts, gchar **errormsg)
               "Extcap [%s] - Cleaning up fifo: %s; PID: %d", interface_opts.name,
               interface_opts.extcap_fifo, interface_opts.extcap_pid);
 #ifdef _WIN32
-        if (pipe_h)
+        if (interface_opts.extcap_pipe_h != INVALID_HANDLE_VALUE)
         {
             g_log(LOG_DOMAIN_CAPTURE, G_LOG_LEVEL_DEBUG,
                   "Extcap [%s] - Closing pipe", interface_opts.name);
-            FlushFileBuffers(pipe_h);
-            DisconnectNamedPipe(pipe_h);
-            CloseHandle(pipe_h);
+            FlushFileBuffers(interface_opts.extcap_pipe_h);
+            DisconnectNamedPipe(interface_opts.extcap_pipe_h);
+            CloseHandle(interface_opts.extcap_pipe_h);
+            interface_opts.extcap_pipe_h = INVALID_HANDLE_VALUE;
+        }
+        if (interface_opts.extcap_control_in_h != INVALID_HANDLE_VALUE)
+        {
+            g_log(LOG_DOMAIN_CAPTURE, G_LOG_LEVEL_DEBUG,
+                  "Extcap [%s] - Closing control_in pipe", interface_opts.name);
+            FlushFileBuffers(interface_opts.extcap_control_in_h);
+            DisconnectNamedPipe(interface_opts.extcap_control_in_h);
+            CloseHandle(interface_opts.extcap_control_in_h);
+            interface_opts.extcap_control_in_h = INVALID_HANDLE_VALUE;
+        }
+        if (interface_opts.extcap_control_out_h != INVALID_HANDLE_VALUE)
+        {
+            g_log(LOG_DOMAIN_CAPTURE, G_LOG_LEVEL_DEBUG,
+                  "Extcap [%s] - Closing control_out pipe", interface_opts.name);
+            FlushFileBuffers(interface_opts.extcap_control_out_h);
+            DisconnectNamedPipe(interface_opts.extcap_control_out_h);
+            CloseHandle(interface_opts.extcap_control_out_h);
+            interface_opts.extcap_control_out_h = INVALID_HANDLE_VALUE;
         }
 #else
         if (interface_opts.extcap_fifo != NULL && file_exists(interface_opts.extcap_fifo))
@@ -1285,7 +1304,7 @@ GPtrArray *extcap_prepare_arguments(interface_options interface_opts)
                     const gchar *stored = NULL, * defval = NULL;
                     /* In case of boolflags only first element in arg_list is relevant. */
                     arg_iter = (extcap_arg *)(arg_list->data);
-                    if (arg_iter->pref_valptr != NULL)
+                    if (arg_iter->pref_valptr != NULL && strlen(*arg_iter->pref_valptr) > 0)
                     {
                         stored = *arg_iter->pref_valptr;
                     }
@@ -1362,15 +1381,27 @@ extcap_init_interfaces(capture_options *capture_opts)
         /* create control pipes if having toolbar */
         if (extcap_has_toolbar(interface_opts.name))
         {
-            extcap_create_pipe(&interface_opts.extcap_control_in);
-            extcap_create_pipe(&interface_opts.extcap_control_out);
+            extcap_create_pipe(interface_opts.name, &interface_opts.extcap_control_in,
+                               EXTCAP_CONTROL_IN_PREFIX, FALSE);
+#ifdef _WIN32
+            interface_opts.extcap_control_in_h = pipe_h;
+#endif
+            extcap_create_pipe(interface_opts.name, &interface_opts.extcap_control_out,
+                               EXTCAP_CONTROL_OUT_PREFIX, FALSE);
+#ifdef _WIN32
+            interface_opts.extcap_control_out_h = pipe_h;
+#endif
         }
 
         /* create pipe for fifo */
-        if (!extcap_create_pipe(&interface_opts.extcap_fifo))
+        if (!extcap_create_pipe(interface_opts.name, &interface_opts.extcap_fifo,
+                                EXTCAP_PIPE_PREFIX, TRUE))
         {
             return FALSE;
         }
+#ifdef _WIN32
+        interface_opts.extcap_pipe_h = pipe_h;
+#endif
 
         /* Create extcap call */
         args = extcap_prepare_arguments(interface_opts);
@@ -1405,9 +1436,19 @@ extcap_init_interfaces(capture_options *capture_opts)
          */
         if (pid != INVALID_EXTCAP_PID)
         {
-            extcap_wait_for_pipe(pipe_h, pid);
-        }
+            HANDLE pipe_handles[3];
+            int num_pipe_handles = 1;
+            pipe_handles[0] = interface_opts.extcap_pipe_h;
 
+            if (extcap_has_toolbar(interface_opts.name))
+            {
+                pipe_handles[1] = interface_opts.extcap_control_in_h;
+                pipe_handles[2] = interface_opts.extcap_control_out_h;
+                num_pipe_handles += 2;
+             }
+
+            extcap_wait_for_pipe(pipe_handles, num_pipe_handles, pid);
+        }
 #endif
 
         interface_opts.extcap_userdata = (gpointer) userdata;
@@ -1419,24 +1460,14 @@ extcap_init_interfaces(capture_options *capture_opts)
     return TRUE;
 }
 
-#ifdef _WIN32
-/* called by capture_sync to get the CreatNamedPipe handle*/
-HANDLE
-extcap_get_win32_handle()
-{
-    return pipe_h;
-}
-#endif
-
-gboolean extcap_create_pipe(char **fifo)
+gboolean extcap_create_pipe(const gchar *ifname, gchar **fifo, const gchar *pipe_prefix, gboolean byte_mode _U_)
 {
 #ifdef _WIN32
     gchar timestr[ 14 + 1 ];
     time_t current_time;
-
     gchar *pipename = NULL;
-
     SECURITY_ATTRIBUTES security;
+
     /* create pipename */
     current_time = time(NULL);
     /*
@@ -1444,7 +1475,7 @@ gboolean extcap_create_pipe(char **fifo)
      * so we won't get a null pointer back from localtime().
      */
     strftime(timestr, sizeof(timestr), "%Y%m%d%H%M%S", localtime(&current_time));
-    pipename = g_strconcat("\\\\.\\pipe\\", EXTCAP_PIPE_PREFIX, "_", timestr, NULL);
+    pipename = g_strconcat("\\\\.\\pipe\\", pipe_prefix, "_", ifname, "_", timestr, NULL);
 
     /* Security struct to enable Inheritable HANDLE */
     memset(&security, 0, sizeof(SECURITY_ATTRIBUTES));
@@ -1452,18 +1483,19 @@ gboolean extcap_create_pipe(char **fifo)
     security.bInheritHandle = TRUE;
     security.lpSecurityDescriptor = NULL;
 
-    /* create a namedPipe*/
+    /* create a namedPipe */
     pipe_h = CreateNamedPipe(
                  utf_8to16(pipename),
                  PIPE_ACCESS_DUPLEX | FILE_FLAG_OVERLAPPED,
-                 PIPE_TYPE_MESSAGE | PIPE_READMODE_MESSAGE | PIPE_WAIT,
-                 5, 65536, 65536,
+                 byte_mode ? PIPE_TYPE_BYTE : PIPE_TYPE_MESSAGE | byte_mode ? PIPE_READMODE_BYTE : PIPE_READMODE_MESSAGE | PIPE_WAIT,
+                 1, 65536, 65536,
                  300,
                  &security);
 
     if (pipe_h == INVALID_HANDLE_VALUE)
     {
         g_log(LOG_DOMAIN_CAPTURE, G_LOG_LEVEL_DEBUG, "\nError creating pipe => (%d)", GetLastError());
+        g_free (pipename);
         return FALSE;
     }
     else
@@ -1475,10 +1507,13 @@ gboolean extcap_create_pipe(char **fifo)
     gchar *temp_name = NULL;
     int fd = 0;
 
-    if ((fd = create_tempfile(&temp_name, EXTCAP_PIPE_PREFIX, NULL)) < 0)
+    gchar *pfx = g_strconcat(pipe_prefix, "_", ifname, NULL);
+    if ((fd = create_tempfile(&temp_name, pfx, NULL)) < 0)
     {
+        g_free(pfx);
         return FALSE;
     }
+    g_free(pfx);
 
     ws_close(fd);
 
