@@ -255,7 +255,15 @@ QWidget *InterfaceToolbar::createSelector(iface_toolbar_control *control)
         combobox->addItem(display, value);
         if (val->is_default)
         {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
             combobox->setCurrentText(display);
+#else
+            int new_index = combobox->findText(display);
+            if (new_index >= 0)
+            {
+                combobox->setCurrentIndex(new_index);
+            }
+#endif
             setDefaultValue(control->num, value.toUtf8());
         }
         foreach (QString ifname, interface_.keys())
@@ -702,7 +710,8 @@ void InterfaceToolbar::closeLog()
     }
 }
 
-void InterfaceToolbar::startReaderThread(QString ifname, QString control_in)
+
+void InterfaceToolbar::startReaderThread(QString ifname, void *control_in)
 {
     QThread *thread = new QThread;
     InterfaceToolbarReader *reader = new InterfaceToolbarReader(ifname, control_in);
@@ -749,20 +758,34 @@ void InterfaceToolbar::startCapture(GArray *ifaces)
             // Already have control channels for this interface
             continue;
 
-        // The reader thread will open control in channel
-        startReaderThread(ifname, interface_opts.extcap_control_in);
-
         // Open control out channel
+#ifdef _WIN32
+        startReaderThread(ifname, interface_opts.extcap_control_in_h);
+        interface_[ifname].out_fd = _open_osfhandle((intptr_t)interface_opts.extcap_control_out_h, O_APPEND | O_BINARY);
+#else
+        startReaderThread(ifname, interface_opts.extcap_control_in);
         interface_[ifname].out_fd = ws_open(interface_opts.extcap_control_out, O_WRONLY | O_BINARY, 0);
-
+#endif
         sendChangedValues(ifname);
         controlSend(ifname, 0, commandControlInitialized);
     }
 
     if (!selected_found && !first_capturing_ifname.isEmpty())
+    {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
         ui->interfacesComboBox->setCurrentText(first_capturing_ifname);
+#else
+        int new_index = ui->interfacesComboBox->findText(first_capturing_ifname);
+        if (new_index >= 0)
+        {
+            ui->interfacesComboBox->setCurrentIndex(new_index);
+        }
+#endif
+    }
     else
+    {
         updateWidgets();
+    }
 #endif // HAVE_EXTCAP
 }
 
@@ -780,7 +803,9 @@ void InterfaceToolbar::stopCapture()
 
         if (interface_[ifname].out_fd != -1)
         {
+#ifndef _WIN32
             ws_close (interface_[ifname].out_fd);
+#endif
             interface_[ifname].out_fd = -1;
         }
 
@@ -935,7 +960,15 @@ void InterfaceToolbar::interfaceListChanged()
             if (selected_ifname.compare(device.name) == 0)
             {
                 // Keep selected interface
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
                 ui->interfacesComboBox->setCurrentText(device.name);
+#else
+                int new_index = ui->interfacesComboBox->findText(device.name);
+                if (new_index >= 0)
+                {
+                    ui->interfacesComboBox->setCurrentIndex(new_index);
+                }
+#endif
                 keep_selected = true;
             }
         }
